# coding:utf-8

__project__ = "xser"
__version__ = "0.5.1"
__urlhome__ = "https://github.com/bondbox/xser/"
__description__ = "Socket and HTTP Server"

# author
__author__ = "Mingzhe Zou"
__author_email__ = "zoumingzhe@outlook.com"
